@@ -1,6 +1,7 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 import datetime
+from itertools import chain
 from decimal import Decimal
 from trytond.model import Workflow, ModelView, ModelSQL, fields
 from trytond.modules.company import CompanyReport
@@ -10,11 +11,10 @@ from trytond.backend import TableHandler
 from trytond.pyson import Eval, Bool, If, PYSONEncoder, Id
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
-from trytond.config import CONFIG
 
 __all__ = ['Purchase', 'PurchaseInvoice', 'PurchaseIgnoredInvoice',
     'PurchaseRecreadtedInvoice', 'PurchaseLine', 'PurchaseLineTax',
-    'PurchaseLineInvoiceLine', 'PurchaseLineIgnoredMove',
+    'PurchaseLineIgnoredMove',
     'PurchaseLineRecreatedMove', 'PurchaseReport', 'Template', 'Product',
     'ProductSupplier', 'ProductSupplierPrice', 'ShipmentIn',
     'ShipmentInReturn', 'Move', 'OpenSupplier', 'HandleShipmentExceptionAsk',
@@ -141,14 +141,14 @@ class Purchase(Workflow, ModelSQL, ModelView):
         cls._order.insert(0, ('purchase_date', 'DESC'))
         cls._order.insert(1, ('id', 'DESC'))
         cls._error_messages.update({
-                'invoice_addresse_required': 'Invoice addresses must be '
-                'defined for the quotation.',
-                'warehouse_required': 'A warehouse must be defined for the ' \
-                    'quotation.',
-                'missing_account_payable': 'It misses ' \
-                        'an "Account Payable" on the party "%s"!',
-                'delete_cancel': 'Purchase "%s" must be cancelled before '\
-                    'deletion!',
+                'invoice_address_required': ('Invoice address must be '
+                    'defined for quotation of sale "%s".'),
+                'warehouse_required': ('A warehouse must be defined for '
+                    'quotation of sale "%s".'),
+                'missing_account_payable': ('Missing "Account Payable" on '
+                    'party "%s".'),
+                'delete_cancel': ('Purchase "%s" must be cancelled before '
+                    'deletion.'),
                 })
         cls._transitions |= set((
                 ('draft', 'quotation'),
@@ -195,16 +195,16 @@ class Purchase(Workflow, ModelSQL, ModelView):
     def __register__(cls, module_name):
         cursor = Transaction().cursor
         # Migration from 1.2: packing renamed into shipment
-        cursor.execute("UPDATE ir_model_data "\
-                "SET fs_id = REPLACE(fs_id, 'packing', 'shipment') "\
-                "WHERE fs_id like '%%packing%%' AND module = %s",
-                (module_name,))
-        cursor.execute("UPDATE ir_model_field "\
-                "SET relation = REPLACE(relation, 'packing', 'shipment'), "\
-                    "name = REPLACE(name, 'packing', 'shipment') "
-                "WHERE (relation like '%%packing%%' "\
-                    "OR name like '%%packing%%') AND module = %s",
-                (module_name,))
+        cursor.execute("UPDATE ir_model_data "
+            "SET fs_id = REPLACE(fs_id, 'packing', 'shipment') "
+            "WHERE fs_id like '%%packing%%' AND module = %s",
+            (module_name,))
+        cursor.execute("UPDATE ir_model_field "
+            "SET relation = REPLACE(relation, 'packing', 'shipment'), "
+                "name = REPLACE(name, 'packing', 'shipment') "
+            "WHERE (relation like '%%packing%%' "
+                "OR name like '%%packing%%') AND module = %s",
+            (module_name,))
         table = TableHandler(cursor, cls, module_name)
         table.column_rename('packing_state', 'shipment_state')
 
@@ -212,9 +212,9 @@ class Purchase(Workflow, ModelSQL, ModelView):
 
         # Migration from 1.2: rename packing to shipment in
         # invoice_method values
-        cursor.execute("UPDATE " + cls._table + " "\
-                "SET invoice_method = 'shipment' "\
-                "WHERE invoice_method = 'packing'")
+        cursor.execute("UPDATE " + cls._table + " "
+            "SET invoice_method = 'shipment' "
+            "WHERE invoice_method = 'packing'")
 
         table = TableHandler(cursor, cls, module_name)
         # Migration from 2.2: warehouse is no more required
@@ -333,10 +333,11 @@ class Purchase(Workflow, ModelSQL, ModelView):
         return 2
 
     def on_change_with_party_lang(self, name=None):
+        Config = Pool().get('ir.configuration')
         if self.party:
             if self.party.lang:
                 return self.party.lang.code
-        return CONFIG['language']
+        return Config.get_language()
 
     def get_tax_context(self):
         context = {}
@@ -456,21 +457,20 @@ class Purchase(Workflow, ModelSQL, ModelView):
                     'invoice_state': state,
                     })
 
-    def get_shipments_returns(attribute):
+    def get_shipments_returns(model_name):
         "Computes the returns or shipments"
         def method(self, name):
-            shipments = []
+            Model = Pool().get(model_name)
+            shipments = set()
             for line in self.lines:
                 for move in line.moves:
-                    ship_or_return = getattr(move, attribute)
-                    if bool(ship_or_return):
-                        if ship_or_return.id not in shipments:
-                            shipments.append(ship_or_return.id)
-            return shipments
+                    if isinstance(move.shipment, Model):
+                        shipments.add(move.shipment.id)
+            return list(shipments)
         return method
 
-    get_shipments = get_shipments_returns('shipment_in')
-    get_shipment_returns = get_shipments_returns('shipment_in_return')
+    get_shipments = get_shipments_returns('stock.shipment.in')
+    get_shipment_returns = get_shipments_returns('stock.shipment.in.return')
 
     def get_moves(self, name):
         return [m.id for l in self.lines for m in l.moves]
@@ -530,12 +530,12 @@ class Purchase(Workflow, ModelSQL, ModelView):
 
     def check_for_quotation(self):
         if not self.invoice_address:
-            self.raise_user_error('invoice_addresse_required')
+            self.raise_user_error('invoice_address_required', (self.rec_name,))
         for line in self.lines:
             if (not line.to_location
                     and line.product
                     and line.product.type in ('goods', 'assets')):
-                self.raise_user_error('warehouse_required')
+                self.raise_user_error('warehouse_required', (self.rec_name,))
 
     @classmethod
     def set_reference(cls, purchases):
@@ -619,7 +619,6 @@ class Purchase(Workflow, ModelSQL, ModelView):
         '''
         pool = Pool()
         Invoice = pool.get('account.invoice')
-        PurchaseLine = pool.get('purchase.line')
 
         if self.invoice_method == 'manual':
             return
@@ -633,18 +632,9 @@ class Purchase(Workflow, ModelSQL, ModelView):
             return
 
         invoice = self._get_invoice_purchase(invoice_type)
+        invoice.lines = list(chain.from_iterable(invoice_lines.itervalues()))
         invoice.save()
 
-        for line in self.lines:
-            if line.id not in invoice_lines:
-                continue
-            for invoice_line in invoice_lines[line.id]:
-                invoice_line.invoice = invoice.id
-                invoice_line.save()
-                PurchaseLine.write([line], {
-                        'invoice_lines': [('add', [invoice_line.id])],
-                        })
-
         with Transaction().set_user(0, set_context=True):
             Invoice.update_taxes([invoice])
 
@@ -872,14 +862,17 @@ class PurchaseLine(ModelSQL, ModelView):
     description = fields.Text('Description', size=None, required=True)
     note = fields.Text('Note')
     taxes = fields.Many2Many('purchase.line-account.tax',
-        'line', 'tax', 'Taxes', domain=[('parent', '=', None)],
+        'line', 'tax', 'Taxes',
+        domain=[('parent', '=', None), ['OR',
+                ('group', '=', None),
+                ('group.kind', 'in', ['purchase', 'both'])],
+            ],
         states={
             'invisible': Eval('type') != 'line',
             }, depends=['type'])
-    invoice_lines = fields.Many2Many('purchase.line-account.invoice.line',
-            'purchase_line', 'invoice_line', 'Invoice Lines', readonly=True)
-    moves = fields.One2Many('stock.move', 'purchase_line', 'Moves',
-            readonly=True)
+    invoice_lines = fields.One2Many('account.invoice.line', 'origin',
+        'Invoice Lines', readonly=True)
+    moves = fields.One2Many('stock.move', 'origin', 'Moves', readonly=True)
     moves_ignored = fields.Many2Many('purchase.line-ignored-stock.move',
             'purchase_line', 'move', 'Ignored Moves', readonly=True)
     moves_recreated = fields.Many2Many('purchase.line-recreated-stock.move',
@@ -892,8 +885,14 @@ class PurchaseLine(ModelSQL, ModelView):
     to_location = fields.Function(fields.Many2One('stock.location',
             'To Location'), 'get_to_location')
     delivery_date = fields.Function(fields.Date('Delivery Date',
-            on_change_with=['product', '_parent_purchase.purchase_date',
-                '_parent_purchase.party']),
+            on_change_with=['product', 'quantity',
+                '_parent_purchase.purchase_date', '_parent_purchase.party'],
+            states={
+                'invisible': ((Eval('type') != 'line')
+                    | (If(Bool(Eval('quantity')), Eval('quantity', 0), 0)
+                        <= 0)),
+                },
+            depends=['type', 'quantity']),
         'on_change_with_delivery_date')
 
     @classmethod
@@ -901,12 +900,13 @@ class PurchaseLine(ModelSQL, ModelView):
         super(PurchaseLine, cls).__setup__()
         cls._order.insert(0, ('sequence', 'ASC'))
         cls._error_messages.update({
-            'supplier_location_required': 'The supplier location is required!',
-            'missing_account_expense': 'It misses ' \
-                    'an "Account Expense" on product "%s"!',
-            'missing_account_expense_property': 'It misses ' \
-                    'an "account expense" default property!',
-            })
+                'supplier_location_required': ('Purchase "%(purchase)s" misses '
+                    'the supplier location for line "%(line)s".'),
+                'missing_account_expense': ('Product "%(product)s" of purchase '
+                    '%(purchase)s misses an expense account.'),
+                'missing_account_expense_property': ('Purchase "%(purchase)s" '
+                    'misses an "account expense" default property.'),
+                })
 
     @classmethod
     def __register__(cls, module_name):
@@ -916,8 +916,8 @@ class PurchaseLine(ModelSQL, ModelView):
 
         # Migration from 1.0 comment change into note
         if table.column_exist('comment'):
-            cursor.execute('UPDATE "' + cls._table + '" ' \
-                    'SET note = comment')
+            cursor.execute('UPDATE "' + cls._table + '" '
+                'SET note = comment')
             table.drop_column('comment', exception=True)
 
         # Migration from 2.4: drop required on sequence
@@ -1097,12 +1097,17 @@ class PurchaseLine(ModelSQL, ModelView):
 
     def on_change_with_delivery_date(self, name=None):
         if (self.product
+                and self.quantity > 0
                 and self.purchase and self.purchase.party
                 and self.product.product_suppliers):
             date = self.purchase.purchase_date if self.purchase else None
             for product_supplier in self.product.product_suppliers:
                 if product_supplier.party == self.purchase.party:
-                    return product_supplier.compute_supply_date(date=date)
+                    delivery_date = product_supplier.compute_supply_date(
+                        date=date)
+                    if delivery_date == datetime.date.max:
+                        return None
+                    return delivery_date
 
     def get_invoice_line(self, invoice_type):
         '''
@@ -1162,15 +1167,19 @@ class PurchaseLine(ModelSQL, ModelView):
         if self.product:
             invoice_line.account = self.product.account_expense_used
             if not invoice_line.account:
-                self.raise_user_error('missing_account_expense',
-                        error_args=(self.product.rec_name,))
+                self.raise_user_error('missing_account_expense', {
+                        'product': invoice_line.product.rec_name,
+                        'purchase': invoice_line.purchase.rec_name,
+                        })
         else:
             for model in ('product.template', 'product.category'):
                 invoice_line.account = Property.get('account_expense', model)
                 if invoice_line.account:
                     break
             if not invoice_line.account:
-                self.raise_user_error('missing_account_expense_property')
+                self.raise_user_error('missing_account_expense_property',
+                    (invoice_line.purchase.rec_name,))
+        invoice_line.origin = self
         return [invoice_line]
 
     @classmethod
@@ -1207,7 +1216,10 @@ class PurchaseLine(ModelSQL, ModelView):
         if quantity <= 0.0:
             return
         if not self.purchase.party.supplier_location:
-            self.raise_user_error('supplier_location_required')
+            self.raise_user_error('supplier_location_required', {
+                    'purchase': self.purchase.rec_name,
+                    'line': self.rec_name,
+                    })
         with Transaction().set_user(0, set_context=True):
             move = Move()
         move.quantity = quantity
@@ -1220,6 +1232,7 @@ class PurchaseLine(ModelSQL, ModelView):
         move.unit_price = self.unit_price
         move.currency = self.purchase.currency
         move.planned_date = self.delivery_date
+        move.origin = self
         return move
 
     def create_move(self):
@@ -1230,10 +1243,6 @@ class PurchaseLine(ModelSQL, ModelView):
         if not move:
             return
         move.save()
-        with Transaction().set_user(0, set_context=True):
-            self.write([self], {
-                    'moves': [('add', [move.id])],
-                    })
         return move
 
 
@@ -1248,16 +1257,6 @@ class PurchaseLineTax(ModelSQL):
             select=True, required=True, domain=[('parent', '=', None)])
 
 
-class PurchaseLineInvoiceLine(ModelSQL):
-    'Purchase Line - Invoice Line'
-    __name__ = 'purchase.line-account.invoice.line'
-    _table = 'purchase_line_invoice_lines_rel'
-    purchase_line = fields.Many2One('purchase.line', 'Purchase Line',
-            ondelete='CASCADE', select=True, required=True)
-    invoice_line = fields.Many2One('account.invoice.line', 'Invoice Line',
-            ondelete='RESTRICT', select=True, required=True)
-
-
 class PurchaseLineIgnoredMove(ModelSQL):
     'Purchase Line - Ignored Move'
     __name__ = 'purchase.line-ignored-stock.move'
@@ -1306,9 +1305,9 @@ class Template:
     def __setup__(cls):
         super(Template, cls).__setup__()
         cls._error_messages.update({
-                'change_purchase_uom': 'Purchase prices are based ' \
-                    'on the purchase uom, are you sure to change it?',
-            })
+                'change_purchase_uom': ('Purchase prices are based '
+                    'on the purchase uom, are you sure to change it?'),
+                })
         required = ~Eval('account_category') & Eval('purchasable', False)
         if not cls.account_expense.states.get('required'):
             cls.account_expense.states['required'] = required
@@ -1431,8 +1430,7 @@ class ProductSupplier(ModelSQL, ModelView):
             ('id', If(Eval('context', {}).contains('company'), '=', '!='),
                 Eval('context', {}).get('company', 0)),
             ])
-    delivery_time = fields.Integer('Delivery Time', required=True,
-            help="In number of days")
+    delivery_time = fields.Integer('Delivery Time', help="In number of days")
     currency = fields.Many2One('currency.currency', 'Currency', required=True,
         ondelete='RESTRICT')
 
@@ -1473,6 +1471,9 @@ class ProductSupplier(ModelSQL, ModelView):
         # Migration from 2.4: drop required on sequence
         table.not_null_action('sequence', action='remove')
 
+        # Migration from 2.6: drop required on delivery_time
+        table.not_null_action('delivery_time', action='remove')
+
     @staticmethod
     def default_company():
         return Transaction().context.get('company')
@@ -1502,13 +1503,12 @@ class ProductSupplier(ModelSQL, ModelView):
     def compute_supply_date(self, date=None):
         '''
         Compute the supply date for the Product Supplier at the given date
-            and the next supply date
         '''
         Date = Pool().get('ir.date')
 
         if not date:
             date = Date.today()
-        if not self.delivery_time:
+        if self.delivery_time is None:
             return datetime.date.max
         return date + datetime.timedelta(self.delivery_time)
 
@@ -1518,7 +1518,7 @@ class ProductSupplier(ModelSQL, ModelView):
         '''
         Date = Pool().get('ir.date')
 
-        if not self.delivery_time:
+        if self.delivery_time is None:
             return Date.today()
         return date - datetime.timedelta(self.delivery_time)
 
@@ -1557,11 +1557,13 @@ class ShipmentIn:
                 add_remove,
                 cls.incoming_moves.add_remove,
                 ]
+        if 'supplier' not in cls.incoming_moves.depends:
+            cls.incoming_moves.depends.append('supplier')
 
         cls._error_messages.update({
-                'reset_move': 'You cannot reset to draft a move generated '\
-                    'by a purchase.',
-            })
+                'reset_move': ('You cannot reset to draft move "%s" because '
+                    'it was generated by a purchase.'),
+                })
 
     @classmethod
     def write(cls, shipments, vals):
@@ -1587,16 +1589,18 @@ class ShipmentIn:
 
             with Transaction().set_user(0, set_context=True):
                 purchases = Purchase.browse([p.id for p in purchases])
-            Purchase.process(purchases)
+                Purchase.process(purchases)
 
     @classmethod
     @ModelView.button
     @Workflow.transition('draft')
     def draft(cls, shipments):
+        PurchaseLine = Pool().get('purchase.line')
         for shipment in shipments:
             for move in shipment.incoming_moves:
-                if move.state == 'cancel' and move.purchase_line:
-                    cls.raise_user_error('reset_move')
+                if (move.state == 'cancel'
+                        and isinstance(move.origin, PurchaseLine)):
+                    cls.raise_user_error('reset_move', (move.rec_name,))
 
         return super(ShipmentIn, cls).draft(shipments)
 
@@ -1608,8 +1612,8 @@ class ShipmentInReturn:
     def __setup__(cls):
         super(ShipmentInReturn, cls).__setup__()
         cls._error_messages.update({
-                'reset_move': 'You cannot reset to draft a move generated '\
-                    'by a purchase.',
+                'reset_move': ('You cannot reset to draft a move generated '
+                    'by a purchase.'),
                 })
 
     @classmethod
@@ -1635,15 +1639,17 @@ class ShipmentInReturn:
 
             with Transaction().set_user(0, set_context=True):
                 purchases = Purchase.browse([p.id for p in purchases])
-            Purchase.process(purchases)
+                Purchase.process(purchases)
 
     @classmethod
     @ModelView.button
     @Workflow.transition('draft')
     def draft(cls, shipments):
+        PurchaseLine = Pool().get('purchase.line')
         for shipment in shipments:
             for move in shipment.moves:
-                if move.state == 'cancel' and move.purchase_line:
+                if (move.state == 'cancel'
+                        and isinstance(move.origin, PurchaseLine)):
                     cls.raise_user_error('reset_move')
 
         return super(ShipmentInReturn, cls).draft(shipments)
@@ -1651,15 +1657,11 @@ class ShipmentInReturn:
 
 class Move(ModelSQL, ModelView):
     __name__ = 'stock.move'
-    purchase_line = fields.Many2One('purchase.line', 'Purchase Line',
-        select=True, states={
-            'readonly': Eval('state') != 'draft',
-            },
-        depends=['state'])
     purchase = fields.Function(fields.Many2One('purchase.purchase', 'Purchase',
-            select=True, states={
+            states={
                 'invisible': ~Eval('purchase_visible', False),
-                }, depends=['purchase_visible']), 'get_purchase',
+                },
+            depends=['purchase_visible']), 'get_purchase',
         searcher='search_purchase')
     purchase_quantity = fields.Function(fields.Float('Purchase Quantity',
             digits=(16, Eval('unit_digits', 2)),
@@ -1684,38 +1686,62 @@ class Move(ModelSQL, ModelView):
                 }, depends=['purchase_visible']), 'get_purchase_fields')
     purchase_visible = fields.Function(fields.Boolean('Purchase Visible',
         on_change_with=['from_location']), 'on_change_with_purchase_visible')
-    supplier = fields.Function(fields.Many2One('party.party', 'Supplier',
-        select=True), 'get_supplier', searcher='search_supplier')
+    supplier = fields.Function(fields.Many2One('party.party', 'Supplier'),
+        'get_supplier', searcher='search_supplier')
     purchase_exception_state = fields.Function(fields.Selection([
         ('', ''),
         ('ignored', 'Ignored'),
         ('recreated', 'Recreated'),
         ], 'Exception State'), 'get_purchase_exception_state')
 
-    def get_purchase(self, name):
-        if self.purchase_line:
-            return self.purchase_line.purchase.id
+    @classmethod
+    def __register__(cls, module_name):
+        cursor = Transaction().cursor
 
-    def get_purchase_exception_state(self, name):
-        if not self.purchase_line:
-            return ''
-        if self in self.purchase_line.moves_recreated:
-            return 'recreated'
-        if self in self.purchase_line.moves_ignored:
-            return 'ignored'
+        super(Move, cls).__register__(module_name)
+
+        table = TableHandler(cursor, cls, module_name)
+
+        # Migration from 2.6: remove purchase_line
+        if table.column_exist('purchase_line'):
+            cursor.execute('UPDATE "' + cls._table + '" '
+                'SET origin = \'purchase.line,\' || purchase_line '
+                'WHERE purchase_line IS NOT NULL')
+            table.drop_column('purchase_line')
+
+    @classmethod
+    def _get_origin(cls):
+        models = super(Move, cls)._get_origin()
+        models.append('purchase.line')
+        return models
+
+    def get_purchase(self, name):
+        PurchaseLine = Pool().get('purchase.line')
+        if isinstance(self.origin, PurchaseLine):
+            return self.origin.purchase.id
 
     @classmethod
     def search_purchase(cls, name, clause):
-        return [('purchase_line.' + name,) + tuple(clause[1:])]
+        return [('origin.' + name,) + tuple(clause[1:]) + ('purchase.line',)]
+
+    def get_purchase_exception_state(self, name):
+        PurchaseLine = Pool().get('purchase.line')
+        if not isinstance(self.origin, PurchaseLine):
+            return ''
+        if self in self.origin.moves_recreated:
+            return 'recreated'
+        if self in self.origin.moves_ignored:
+            return 'ignored'
 
     def get_purchase_fields(self, name):
-        if self.purchase_line:
+        PurchaseLine = Pool().get('purchase.line')
+        if isinstance(self.origin, PurchaseLine):
             if name[9:] == 'currency':
-                return self.purchase_line.purchase.currency.id
+                return self.origin.purchase.currency.id
             elif name[9:] in ('quantity', 'unit_digits', 'unit_price'):
-                return getattr(self.purchase_line, name[9:])
+                return getattr(self.origin, name[9:])
             else:
-                return getattr(self.purchase_line, name[9:]).id
+                return getattr(self.origin, name[9:]).id
         else:
             if name[9:] == 'quantity':
                 return 0.0
@@ -1729,12 +1755,14 @@ class Move(ModelSQL, ModelView):
         return False
 
     def get_supplier(self, name):
-        if self.purchase_line:
-            return self.purchase_line.purchase.party.id
+        PurchaseLine = Pool().get('purchase.line')
+        if isinstance(self.origin, PurchaseLine):
+            return self.origin.purchase.party.id
 
     @classmethod
     def search_supplier(cls, name, clause):
-        return [('purchase_line.purchase.party',) + tuple(clause[1:])]
+        return [('origin.purchase.party',) + tuple(clause[1:]) +
+            ('purchase.line',)]
 
     @classmethod
     def write(cls, moves, vals):
@@ -1754,7 +1782,7 @@ class Move(ModelSQL, ModelView):
             if purchases:
                 with Transaction().set_user(0, set_context=True):
                     purchases = Purchase.browse([p.id for p in purchases])
-                Purchase.process(purchases)
+                    Purchase.process(purchases)
 
     @classmethod
     def delete(cls, moves):
@@ -1775,7 +1803,7 @@ class Move(ModelSQL, ModelView):
             if purchases:
                 with Transaction().set_user(0, set_context=True):
                     purchases = Purchase.browse([p.id for p in purchases])
-                Purchase.process(purchases)
+                    Purchase.process(purchases)
 
 
 class OpenSupplier(Wizard):
@@ -1798,7 +1826,6 @@ class OpenSupplier(Wizard):
         model_data, = ModelData.search([
                 ('fs_id', '=', 'act_open_supplier'),
                 ('module', '=', 'purchase'),
-                ('inherit', '=', None),
                 ], limit=1)
         wizard = Wizard(model_data.db_id)
 
@@ -1812,8 +1839,8 @@ class HandleShipmentExceptionAsk(ModelView):
     recreate_moves = fields.Many2Many(
         'stock.move', None, None, 'Recreate Moves',
         domain=[('id', 'in', Eval('domain_moves'))], depends=['domain_moves'],
-        help='The selected moves will be recreated. '\
-            'The other ones will be ignored.')
+        help=('The selected moves will be recreated. '
+            'The other ones will be ignored.'))
     domain_moves = fields.Many2Many(
         'stock.move', None, None, 'Domain Moves')
 
@@ -1821,10 +1848,10 @@ class HandleShipmentExceptionAsk(ModelView):
     def __register__(cls, module_name):
         cursor = Transaction().cursor
         # Migration from 1.2: packing renamed into shipment
-        cursor.execute("UPDATE ir_model "\
-                "SET model = REPLACE(model, 'packing', 'shipment') "\
-                "WHERE model like '%%packing%%' AND module = %s",
-                (module_name,))
+        cursor.execute("UPDATE ir_model "
+            "SET model = REPLACE(model, 'packing', 'shipment') "
+            "WHERE model like '%%packing%%' AND module = %s",
+            (module_name,))
         super(HandleShipmentExceptionAsk, cls).__register__(module_name)
 
 
@@ -1893,8 +1920,8 @@ class HandleInvoiceExceptionAsk(ModelView):
         'account.invoice', None, None, 'Recreate Invoices',
         domain=[('id', 'in', Eval('domain_invoices'))],
         depends=['domain_invoices'],
-        help='The selected invoices will be recreated. '\
-            'The other ones will be ignored.')
+        help=('The selected invoices will be recreated. '
+            'The other ones will be ignored.'))
     domain_invoices = fields.Many2Many(
         'account.invoice', None, None, 'Domain Invoices')
 
